@@ -144,17 +144,29 @@ writeCabalFile names = do
   putStrLn "Modified highlighting-kate.cabal."
   putStrLn "Backed up original as highlighting-kate.cabal.orig."
 
+isIncludeRules :: SyntaxParser -> Bool
+isIncludeRules p =
+  parserType p == "IncludeRules" && "##" `isInfixOf` parserContext p
+
+includeLangs :: SyntaxDefinition -> [String]
+includeLangs syntax =
+  nub $ map (takeLang . parserContext) $
+        filter isIncludeRules $ concatMap contParsers $ synContexts syntax
+
+takeLang :: String -> String
+takeLang [] = []
+takeLang ('#':'#':xs) = xs
+takeLang (_:xs) = takeLang xs
+
 processOneFile :: FilePath -> IO ()
 processOneFile src = do
   [syntax] <- runX $ application src
   let name = nameFromPath src
   let outFile = joinPath [libraryPath, "Syntax", addExtension name "hs"]
-  let isIncludeRules p = parserType p == "IncludeRules" && "##" `isPrefixOf`
-                              parserContext p
-  let includeLangs = nub $ filter (/= name) $ map (drop 2 . parserContext) $
-        filter isIncludeRules $ concatMap contParsers $ synContexts syntax
+  -- let includeLangs = nub $ filter (/= name) $ map (drop 2 . parserContext) $
+  --      filter isIncludeRules $ concatMap contParsers $ synContexts syntax
   let includeImports = map (("import qualified " ++) . langNameToModule)
-                        includeLangs
+                        (filter (/= name) $ includeLangs syntax)
   putStrLn $ "Writing " ++ outFile
   B.writeFile outFile $ fromString $
            "{- This module was generated from data in the Kate syntax\n\
@@ -206,9 +218,10 @@ mkParser syntax =
                    text "                        , synStPrevNonspace = synStPrevNonspace st || not (all isSpace txt) }" $$
                    text "return (attr, txt)")
       parseExpression = text "-- | Parse an expression using appropriate local context." $$
-                        text "parseExpression :: KateParser Token" $$
-                        text "parseExpression = do" $$ (nest 2 $
-                          text "(lang,cont) <- currentContext" $$
+                        text "parseExpression :: Maybe (String,String)" $$
+                        text "                -> KateParser Token" $$
+                        text "parseExpression mbcontext = do" $$ (nest 2 $
+                          text "(lang,cont) <- maybe currentContext return mbcontext" $$
                           text ("result <- parseRules (lang,cont)") $$
                           text "optional $ do eof" $$
                           text "              updateState $ \\st -> st{ synStPrevChar = '\\n' }" $$
@@ -220,12 +233,10 @@ mkParser syntax =
       --   text $ "lineBeginContexts = " ++ (show $ map (\cont -> (contName cont, contLineBeginContext cont)) $ synContexts syntax)
       startingContext = head (synContexts syntax)
       -- contextNull = text $ "parseRules \"\" = parseRules " ++ show (synLanguage syntax, contName startingContext)
-      isIncludeRules p = parserType p == "IncludeRules" && "##" `isPrefixOf`
-                              parserContext p
-      includeLangs = nub $ map (drop 2 . parserContext) $
-                     filter isIncludeRules $
-                     concatMap contParsers $ synContexts syntax
-      foreignContexts = vcat $ map (\l -> text ("parseRules (" ++ show l ++ ", _) = " ++ langNameToModule l ++ ".parseExpression")) includeLangs
+      -- includeLangs = nub $ map (drop 2 . parserContext) $
+      --                filter isIncludeRules $
+      --                concatMap contParsers $ synContexts syntax
+      foreignContexts = vcat $ map (\l -> text ("parseRules (" ++ show l ++ ", _) = " ++ langNameToModule l ++ ".parseExpression Nothing")) (includeLangs syntax)
       contextCatchAll = text $ "parseRules x = parseRules " ++ show (synLanguage syntax, contName startingContext) ++ " <|> fail (\"Unknown context\" ++ show x)"
       contexts = map (mkRules syntax) $ synContexts syntax
       initialContextStack = [(synLanguage syntax, contName startingContext)]
@@ -241,7 +252,7 @@ mkParser syntax =
                             \highlight :: String -> [SourceLine]\n\
                             \highlight input = evalState (mapM parseSourceLine $ lines input) startingState"
       lineParser = text   $ "parseSourceLine :: String -> State SyntaxState SourceLine\n\
-                            \parseSourceLine = mkParseSourceLine parseExpression"
+                            \parseSourceLine = mkParseSourceLine (parseExpression Nothing)"
       endLineParser = text "pEndLine = do" $$
                       (nest 2 $ text "updateState $ \\st -> st{ synStPrevNonspace = False }" $$
                                 text "context <- currentContext" $$
@@ -315,12 +326,15 @@ mkSyntaxParser syntax context parser =
             "HlCChar"          -> "pHlCChar"
             "RangeDetect"      -> "pRangeDetect " ++ show (parserChar parser) ++ " " ++ show (parserChar1 parser)
             "LineContinue"     -> "pLineContinue"
-            "IncludeRules"     -> case parserContext parser of
-                                      ('#':'#':xs) -> langNameToModule xs ++ ".parseExpression" ++
-                                                      if parserIncludeAttrib parser || attr' == NormalTok
-                                                         then ""
-                                                         else " >>= ((withAttribute " ++ show attr' ++ ") . snd)"
-                                      xs           -> "parseRules " ++ show (synLanguage syntax, xs)
+            "IncludeRules"     ->
+                  case break (=='#') $ parserContext parser of
+                       (cont,'#':'#':lang) ->
+                          langNameToModule lang ++ ".parseExpression (" ++
+                          show (Just (lang, cont)) ++
+                          if parserIncludeAttrib parser || attr' == NormalTok
+                             then ")"
+                             else ") >>= ((withAttribute " ++ show attr' ++ ") . snd)"
+                       (cont,_)     -> "parseRules " ++ show (synLanguage syntax, cont)
             "DetectSpaces"     -> "pDetectSpaces"
             "DetectIdentifier" -> "pDetectIdentifier"
             _                  -> "pUnimplemented"
